from rallf.sdk.network import Network


class Logger(Network):
    """
        Log severity codes according with RFC 3164
        @see https://www.ietf.org/rfc/rfc3164.txt
    """

    def __init__(self, input, output, context):
        super().__init__(input, output, context)

    def log(self, msg, severity=7, channel="", data=None):
        return self.event(
            "log",
            {
                "message": str(msg),
                "channel": channel,
                "severity": severity,
                "data": data
            }
        )

    def debug(self, msg, channel="", data=None):
        """Debug: debug-level messages"""
        return self.log(msg, 7, channel, data)

    def info(self, msg, channel="", data=None):
        """Informational: informational messages"""
        return self.log(msg, 6, channel, data)

    def notice(self, msg, channel="", data=None):
        """Notice: normal but significant condition"""
        return self.log(msg, 5, channel, data)

    def warning(self, msg, channel="", data=None):
        """Warning: warning conditions"""
        return self.log(msg, 4, channel, data)

    def error(self, msg, channel="", data=None):
        """Error: error conditions"""
        return self.log(msg, 3, channel, data)

    def critical(self, msg, channel="", data=None):
        """Critical: critical conditions"""
        return self.log(msg, 2, channel, data)

    def alert(self, msg, channel="", data=None):
        """Alert: action must be taken immediately"""
        return self.log(msg, 1, channel, data)

    def emergency(self, msg, channel="", data=None):
        """Emergency: system is unusable"""
        return self.log(msg, 0, channel, data)



from rallf.tools.communicator import Communicator


class Network(Communicator):
    def __init__(self, input, output, context):
        super().__init__(input, output)
        self.context = context

    def call(self, routing, target, routine, args):
        return self.rpccall("delegate", {"routing": routing, "target": target, "method": routine, "params": args})

    def use(self, routine, target, args=None):
        return self.call("local", target, routine, args)

    def delegate(self, routine, target, args=None):
        return self.call("remote", target, routine, args)

    def event(self, name, content=None):
        return self.rpccall("event", {"name": name, "context": self.context, "content": content}, wait=False)



import time

from selenium.common.exceptions import WebDriverException

from rallf.sdk import Task
from rallf.tools import SeleniumDeviceFactory


class Title(Task):
    def __init__(self, manifest, robot, input, output):
        super().__init__(manifest, robot, input, output)
        self.browser = None

    def warmup(self):
        self.logger.info("WARMUP!!!")
        factory = SeleniumDeviceFactory(self.robot)
        self.browser = factory.build('firefox63')
        super().warmup()

    def get_title(self, input):
        self.logger.debug("get_title started!")
        self.logger.debug(len(self.robot.devices))
        assert self.browser is not None
        self.browser.get(input['url'])
        time.sleep(10)
        return self.browser.title

    def cooldown(self):
        self.logger.info("cooldown!!!")
        try:
            self.browser.close()
        except WebDriverException:
            self.logger.warning("driver already closed")

        super().cooldown()



import uuid


class Execution:
    def __init__(self, task, func, robot, input):
        self.id = uuid.uuid4()
        self.task = task
        self.func = func
        self.robot = robot
        self.input = input




import time
import base64

from appium.webdriver.common.touch_action import TouchAction
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from rallf.sdk import Task
from rallf.sdk.rallf_error import RallfError
from rallf.tools import SeleniumDeviceFactory
from .image_finder import ImageFinder


class InstallApp(Task):
    """ This Task installs a Google Play App given the icon and the search keywords.
        It scrolls down to find the given icon.
        Tested in Android 7 with resolution 720x1280 (WxH)
    """
    def __init__(self, manifest, robot, input, output):
        super().__init__(manifest, robot, input, output)
        self.device = None
        self.wait = None
        self.wait_for = lambda by, selector: self.wait.until(EC.presence_of_element_located((by, selector)))

        self.kbmap = {
            "special": {
                ' ': {
                    "y": 1226,
                    "x": 400
                },
                '.': {
                    "y": 1226,
                    "x": 573
                },
                "continue": {
                    "y": 1226,
                    "x": 660,
                    "aliases": ["search", "next", "go", "enter"]
                },
                "backspace": {
                    "y": 1126,
                    "x": 660,
                    "aliases": ["delete"]
                },
                "caps": {
                    "y": 1126,
                    "x": 57
                },
                "numbers": {
                    "y": 1226,
                    "x": 57
                },
                "letters": {
                    "y": 1226,
                    "x": 57
                }
            },
            "chars": {
                "1234567890": {
                    "y": 926,
                    "xrange": [40, 680]
                },
                "qwertyuiop": {
                    "y": 926,
                    "xrange": [40, 680]
                },
                "asdfghjkl": {
                    "y": 1026,
                    "xrange": [76, 644]
                },
                "zxcvbnm": {
                    "y": 1126,
                    "xrange": [145, 570]
                }
            }
        }

    def keyboard_write(self, text):
        mode = "lowercase"
        for letter in text:
            if letter.isupper():
                self.device.tap([self.keyboard_coords("caps")])
            if letter.isdigit():
                self.device.tap([self.keyboard_coords("numbers")])
                mode = "numbers"
            self.device.tap([self.keyboard_coords(letter.lower())])
            if mode == "numbers":
                self.device.tap([self.keyboard_coords("letters")])
                mode = "lowercase"

    def keyboard_special(self, special):
        self.device.tap([self.keyboard_coords(special)])

    def keyboard_coords(self, letter):
        special = self.kbmap['special']
        if letter in special.keys():
            return special[letter]['x'], special[letter]['y']

        chars = self.kbmap['chars']
        for key in chars.keys():
            sep = int((chars[key]['xrange'][1] - chars[key]['xrange'][0])/(len(key) - 1))
            if letter in key:
                x = chars[key]['xrange'][0] + key.index(letter) * sep
                return x, chars[key]['y']

    def warmup(self):
        factory = SeleniumDeviceFactory(self.robot)
        self.device = factory.build('android4.4.2')
        self.wait = WebDriverWait(self.device, 10)
        # Go Home
        self.logger.debug("go home")
        self.device.press_keycode(0x3)
        time.sleep(1)
        self.logger.info("WARMUP!!!")
        super().warmup()

    def install(self, input):
        self.logger.debug("install started!")
        icon_file = open("app-logo.png", "wb")
        icon_file.write(base64.b64decode(input['logo']))
        icon_file.flush()
        finder = ImageFinder("app-logo.png")

        # open google play
        self.logger.debug("open google play")
        self.device.tap([(610, 1200)])
        time.sleep(3)

        # tap on search bar
        self.device.tap([(240, 110)])
        time.sleep(1)
        self.keyboard_write(input['search'])
        self.keyboard_special("continue")
        size = self.device.get_window_size()
        self.logger.debug("SIZE: %s" % size)

        scrolls = 0
        while scrolls < 20:
            time.sleep(2)
            self.device.get_screenshot_as_file("screen.png")
            match = finder.match("screen.png")
            self.logger.debug("RES: %s" % match)
            if match['matches'] > 0.99:
                self.logger.debug("APP found!!")
                self.device.tap([match['coords']])
                return
            scrolls += 1
            actions = TouchAction(self.device)
            actions.press(x=int(size['width'] * 0.4), y=int(size['height'] * 0.8))
            actions.wait(1000)
            actions.move_to(x=int(size['width'] * 0.6), y=int(size['height'] * 0.2))
            actions.release()
            actions.perform()

        raise RallfError(32000, "app not found")

    def cooldown(self):
        self.logger.info("cooldown!!!")
        try:
            self.device.close()
        except WebDriverException:
            self.logger.warning("device already closed")



import json
import os


class TaskFactory:

    @staticmethod
    def createFromDir(dir, robot, input, output):
        manifest_path = "%s/config/manifest.json" % dir

        if not os.path.isfile(manifest_path):
            raise FileNotFoundError("manifest.json not found in package")

        manifest = json.load(open(manifest_path, "r"))
        task_class = manifest['main'].split(".")
        package = ".".join(task_class[:-1])
        classname = task_class[-1]
        Task = getattr(__import__(package, fromlist=[classname]), classname)
        return Task(manifest, robot, input, output)



#!/usr/bin/env python3
import json
import signal
import sys
import argparse

from rallf.args import RallfArgs
from rallf.tools.execution import Execution
from rallf.tools.robot_factory import RobotFactory
from rallf.tools.runner import Runner
from rallf.tools.task_factory import TaskFactory


def sigint_handler(sig, frame):
    raise InterruptedError()


def main():
    signal.signal(signal.SIGINT, sigint_handler)

    p = argparse.ArgumentParser(description="Rallf developer tool")
    p.add_argument("command", nargs=1, action="store", choices=["new-task", "run", "event"], help="rallf command")
    p.add_argument("task_dir", action="store", nargs="?", default=".", help="task directory (default: current)")
    p.add_argument("-f", "--func", dest="func", default=None, action="store", help="the routine to execute (default: None)")
    p.add_argument("-r", "--robot", dest="robot", action="store", default=None, help="robot to invoke (default: nullbot)")
    p.add_argument("-m", "--mocks", dest="mocks", action="store", default=None, help="mocks directory (default: None)")
    p.add_argument("-i", "--input", dest="input", default="{}", action="store", help="task's input in JSON (default: {})")

    args = RallfArgs()
    p.parse_args(namespace=args)

    cmd_line = args.getProcessed()

    rf = RobotFactory()

    bot = rf.createEmpty() if cmd_line.robot is None else rf.createFromDir(cmd_line.robot)

    tf = TaskFactory()
    task = tf.createFromDir(cmd_line.task_dir, bot, sys.stdin, sys.stdout)

    if cmd_line.func is not None and cmd_line.func not in task.manifest['exports']:
        raise RuntimeError("%s function not exported in package" % cmd_line.func)

    x = Execution(task, cmd_line.func, bot, json.loads(cmd_line.input))

    Runner.execute(x)


if __name__ == "__main__":
    main()



from rallf.sdk.logger import Logger
from rallf.sdk.network import Network
from rallf.sdk.listener import Listener


class Task:

    def __init__(self, manifest, robot, input, output):
        self.manifest = manifest
        self.robot = robot
        self.finished = False
        self.status = "stopped"
        self.network = Network(input, output, self.manifest['fqtn'])
        self.logger = Logger(input, output, self.manifest['fqtn'])
        self.listener = Listener(input, output)
        self.home = "%s/data/%s" % (robot.home, manifest['fqtn'])

    def warmup(self):
        self.status = "ready"

    def waitloop(self):
        while not self.finished:
            self.listener.listen(self)

    def main(self, input):
        pass

    def cooldown(self):
        self.status = "finished"

    def finish(self):
        self.status = "terminating"
        self.finished = True



import json
import os


class RallfArgs(object):

    def __init__(self):
        self.command = None
        self.task_dir = None
        self.func = None
        self.robot = None
        self.mocks = None
        self.input = "{}"

    def __str__(self):
        return json.dumps({
            "command": self.command,
            "task_dir": self.task_dir,
            "func": self.func,
            "robot": self.robot,
            "mocks": self.mocks,
            "input": self.input,
        })

    def getProcessed(self):
        processed = RallfArgs()
        processed.command = self.command
        processed.task_dir = None if self.task_dir is None else os.path.abspath(self.task_dir)
        processed.func = self.func
        processed.robot = None if self.robot is None else os.path.abspath(self.robot)
        processed.mocks = None if self.mocks is None else os.path.abspath(self.mocks)
        processed.input = self.input
        return processed



import json
import random


class Communicator:
    def __init__(self, input, output):
        self.input = input
        self.output = output
        self.mq = []

    def rpcsend(self, request):
        request.update({"jsonrpc": "2.0"})
        self.output.write("%s\n" % json.dumps(request))

    def rpcreceive(self):
        input = self.input.readline()
        if not input: raise EOFError("End Of File detected")
        return json.loads(input)

    def rpccall(self, method, params, wait=True):
        request = {"jsonrpc": "2.0", "method": method, "params": params}
        if wait is True:
            uniqid = random.randint(0, 9999999)
            self.rpcsend({**request, **{"id": uniqid}})
            return self.waitfor(uniqid)
        self.rpcsend(request)

    def rpcresponse(self, id=None, result=False, error=False):
        if result is False and error is False or result is not False and error is not False:
            raise RuntimeError("Incompatible response: 'result' and 'error' cannot be the same")

        response = {"jsonrpc": "2.0", "id": id}
        if result is not False: response.update({"result": result})
        elif error is not False: response.update({"error": error})

        self.rpcsend(response)

    def wait(self, forcereceive=False):
        if not forcereceive and len(self.mq) > 0: return self.mq.pop(0)
        return self.rpcreceive()

    def waitfor(self, id):
        while True:
            for i in range(len(self.mq)):
                if "method" not in self.mq[i] and self.mq[i]['id'] == id:
                    return self.mq.pop(i)
            input = self.wait(True)
            if "method" not in input and input['id'] == id: return input
            else: self.mq.append(input)



import setuptools

setuptools.setup(
  name="rallf",
  version="0.2.0",
  author="Lluis Santos",
  author_email="lluis@rallf.com",
  license="MIT",
  description="rallf.com Software Development Kit (SDK)",
  long_description=open("README.md", "r").read(),
  long_description_content_type="text/markdown",
  url="https://github.com/robotunion/rallf-python-sdk",
  packages=setuptools.find_packages(),
  scripts=['bin/rallf'],
  classifiers=[
    "Development Status :: 1 - Planning",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Intended Audience :: Developers",
    "Topic :: Software Development",
    "Topic :: Scientific/Engineering :: Artificial Intelligence",
  ],
  install_requires=[pkg.strip() for pkg in open("requirements.txt").readlines() if len(pkg) > 1]
)



from rallf.tools.execution import Execution
import os


class Runner:
    def __init__(self):
        pass

    @staticmethod
    def execute(execution: Execution):
        home = execution.task.home

        if not os.path.exists(home): os.mkdir(home)
        os.chdir(home)
        execution.task.warmup()
        execution.task.network.event("warmup:end")
        if execution.func is None: execution.task.waitloop()
        else: print(getattr(execution.task, execution.func)(execution.input))
        execution.task.cooldown()
